#! /bin/ksh
#
# findHbas.sh
#
# Locates all Fibre Channel bus adpaters in the system and stores
# the WWID and host port number for each HBA found in the file '/tmp/hbas.list'.
#
# Copyright @ 1999, Compaq Computer Corp., All Rights Reserved.
#
# Author: M. Reese
# @version <!--$$Revision:--!> 1 <!-- $-->
#
# History:
#

#---------------------------------------------------------------------------
# Global variables
#---------------------------------------------------------------------------
messagesFile="/var/adm/messages"
osVer=`uname -r`

#---------------------------------------------------------------------------
# Verify that the user has root privileges
#---------------------------------------------------------------------------
checkUID() {
    ## Must run as root
    typeset -i uid
    uid=`id | /usr/bin/sed 's/uid=\([0-9]*\)(.*/\1/'`
    if [ uid -ne 0 ]; then
        echo "You must be root to execute this script!"
        exit 1
    fi
}


#---------------------------------------------------------------------------
# Locate and log the WWIDs for a Solaris system
#---------------------------------------------------------------------------
getSolarisWwids() {
    # determine the month and day of the last reboot
    month=`who -b | cut -d":" -f1 | awk '{print $4}'`
    day=`who -b | cut -d":" -f1 | awk '{print $5}'`

    # determine the instance name for each fibre adapter in the system,
    # storing the results in '/tmp/devices.list'
    findDeviceInstances

    # read the "/tmp/devices.list" file and use the instance name
    # to locate the WWN
    exec 4<&0   # save stdin to file descriptor 4

    while read line; do
        # use the 'instanceName' from "/tmp/devices.list" to get the WWN
        instanceName=`echo $line | cut -d" " -f1`
        #echo "$instanceName"

        #
        # USE cat TO LIST THE /var/adm/messages(.#) FILE
        #
        wwn=`cat /var/adm/message* | grep "$month [ ]*$day" |
        #
        # PIPE EACH LINE INTO nawk TO EXTRACT THE WWN FOR EACH
        # FIBRE ADAPTER
        #
        /usr/bin/nawk -v instance=$instanceName -v osVersion=$osVer '
            /WWPN/ {
                # grab the instance number
                if ( osVersion ~ "5.8" )
                    inst = substr( $9, 1, length($9)-1 )
                else
                    inst = substr( $6, 1, length($6)-1 )

                # if that matches what we are looking for, return the WWN
                if ( inst ~ instance )
                    if ( osVersion ~ "5.8" )
                        wwn = $15
                    else
                        wwn = $12
                else
                    break

                # check if this particular WWN is in the list; if not, add it
                if ( inst in wwnList )
                    inList = 1
                else
                    wwnList[inst] = wwn
            }
            END {
                # print the list
                for ( inst in wwnList ) {
                    printf "%s", wwnList[inst]
                }
            }
        '`
        #echo $wwn

        # use the 'physDeviceName' from "devices.list" to get the host port (bus)
        physDeviceName=`echo $line | cut -d" " -f2`

    # now get the bus number
    bus=`ls -l /dev/dsk |
        #
        # PIPE THE OUTPUT OF THE ls TO nawk TO LOCATE A DEVICE BEHIND
        # THE FIBRE ADAPTER. EXTRACT AND RETURN THE BUS NUMBER FROM
        # THE DEVICE.
        #
    nawk -v dev=$physDeviceName '
            /dev/ {
                pdn = substr( $11, 14, length(dev) )
                if ( dev ~ pdn ) {
                    #printf "%s => %s\n", dev, $9
                    print substr( $9, 2, 1 )
                    exit
                }
            }
        '`

    # Short term Secure Path fix.  If installed, set bus to 99 so
    # this system does not show up in the "mount on host" drop down list box.
    # Check installed packages for Secure Path.
        pkginfo -l|grep CPQswsp>/dev/null
        if [ $? == 0 ]; then
            echo "Determined that Secure Path is Installed."
            bus=99
        fi

    # write the result to the '/tmp/hbas.list' file
        if [ -n "$wwn" -a -n "$bus" ]; then
        echo $wwn $bus >> "/tmp/hbas.list"
        echo "Found WWN: " $wwn $bus
        fi
    done <"/tmp/devices.list"

    exec 4<&-   # close the input file descriptor
}


#---------------------------------------------------------------------------
# Find and store the device instance and physical device name
#---------------------------------------------------------------------------
findDeviceInstances() {
    #
    # USE cat TO LIST THE /etc/path_to_inst FILE
    #
    instances=`cat /etc/path_to_inst |
    #cat /etc/path_to_inst |
    #
    # PIPE THAT INTO grep LOOKING FOR THE FIBRE TAG AND THEN INTO
    # sed TO STRIP OUT DOUBLE QUOTES (")
    grep \"fca | sed 's/\"//g' |
    #
    # PASS THE RESULTING LINE TO awk TO FORMAT THE DESIRED INFO
    # AND SAVE IT TO THE FILE /tmp/devices.list.
    #
    /usr/bin/nawk '
    {
        dev = $3$2
        hbaList[dev] = $1
}
    END {
        for ( dev in hbaList ) {
            printf "%s %s\n", dev, hbaList[dev] > "/tmp/devices.list"
        }
    }
    '`
}


#---------------------------------------------------------------------------
# Locate and log the WWIDs for a Tru64 system
#---------------------------------------------------------------------------
getTru64Wwids()
{
    /usr/sbin/uerf -Rr 300 |
    #
    # PASS EACH LINE TO nawk TO EXTRACT THE WWN AND HOST PORT (BUS)
    #
    /usr/bin/nawk '
/KGPSA-BC/ {
    # read the next one lines
        getline

    # get the WWID
    if ( substr($1, 1, 1) ~ /_/ ) {
        wwid = sprintf( "%s%s%s%s", substr($4, 1, 4), substr($4, 6, 4), substr($4, 11, 4), substr($4, 16, 4) )

        # get the host port
        getline
        while ( substr($1, 1, 4) !~ /scsi/ )
            getline
        port = substr( $1, 5, 1 )
    }

    # check if this particular WWID is in the list; if not, add it
        if ( port in wwidList )
        inList = 1
    else
        wwidList[port] = wwid
}
END {
    # print the list to the screen and to the file
        for ( port in wwidList ) {
      if ( length(wwidList[port]) > 0 ) {
            printf "%s %d\n", wwidList[port], port >> "/tmp/hbas.list"
            printf "WWID = %s, Port = %d\n", wwidList[port], port
      }  # end if
        }    # end for
}' 

} #end getTru64Wwids


#---------------------------------------------------------------------------
# Main
#---------------------------------------------------------------------------

checkUID

## Determine the platform operating system
os=`uname -s`

## Removing any existing "/tmp/hbas.list" and "/tmp/devices.list" file
rm -f /tmp/hbas.list
rm -f /tmp/devices.list

if [ $os = "SunOS" ]; then
    getSolarisWwids
elif [ $os = "OSF1" ]; then
    getTru64Wwids
fi

exit 0
